#version 400

// Interpolated inputs from Vertex Shader
in vec3 eye_normal;
in vec3 eye_vertex;
in vec2 UV;

// Final color of this fragment
out vec4 frag_color;

// Built-in Variables
uniform vec3 point_light_diffuse[4];
uniform vec3 point_light_specular[4];
uniform vec3 point_light_ambient[4];
uniform vec3 point_light_position[4];
uniform float point_light_atten_quad[4];
uniform float point_light_atten_linear[4];
uniform float point_light_atten_const[4];
uniform vec3 dir_light_diffuse[4];
uniform vec3 dir_light_specular[4];
uniform vec3 dir_light_ambient[4];
uniform vec3 dir_light_direction[4];

// User-defined Variables
uniform vec3 EmissiveColor;
uniform vec3 AmbientColor;
uniform vec3 DiffuseColor;
uniform vec3 SpecularColor;
uniform float Shininess;

vec3 DirLightContribution(int light_num) {
	// Directional Light Vectors
	vec3 N = normalize(eye_normal);
	vec3 V = -normalize(eye_vertex);
	vec3 L = -normalize(dir_light_direction[light_num]);
	vec3 H = normalize(V+L);

	// B factor
	float B = 1.0;
	if (dot(N, L) < 0.00001) B = 0.0;

	// Contribution
	float diffuseShade = max(dot(N, L), 0.0);
	float shininess = Shininess > 0 ? Shininess : 0.00001; // Edge case
	float specularShade = B * pow(max(dot(H, N), 0.0), shininess);

	vec3 ambient = AmbientColor * dir_light_ambient[light_num];
	vec3 diffuse = diffuseShade * DiffuseColor * dir_light_diffuse[light_num];
	vec3 specular = specularShade * SpecularColor * dir_light_specular[light_num];

	return ambient + diffuse + specular;
}

vec3 PointLightContribution(int light_num) {
	// Point Light Vectors
	vec3 N = normalize(eye_normal);
	vec3 V = -normalize(eye_vertex); // V = eye_vertex - origin = eye_vertex
	vec3 light_vector = point_light_position[light_num] - eye_vertex;
	vec3 L = normalize(light_vector);
	vec3 H = normalize(V+L);

	// Attenuation
	float r = length(light_vector);
	float attenuation = point_light_atten_const[light_num] + point_light_atten_linear[light_num] * r + point_light_atten_quad[light_num] * r * r;
	float luminosity = 0;
	if (attenuation > 0) luminosity = 1.0 / attenuation;

	// B factor
	float B = 1.0;
	if (dot(N, L) < 0.00001) B = 0.0;

	// Contribution
	float diffuseShade = max(dot(N, L), 0.0);
	float shininess = Shininess > 0 ? Shininess : 0.00001; // Edge case
	float specularShade = B * pow(max(dot(H, N), 0.0), shininess);

	vec3 ambient = AmbientColor * point_light_ambient[light_num];
	vec3 diffuse = diffuseShade * DiffuseColor * point_light_diffuse[light_num] * luminosity;
	// vec3 diffuse = diffuseShade * vec3(texture(DiffuseMap, UV)) * point_light_diffuse[light_num] * luminosity; // DiffuseMap
	vec3 specular = specularShade * SpecularColor * point_light_specular[light_num] * luminosity;
	
	return ambient + diffuse + specular;
}

void main()
{
	// Assign final color
	vec3 dirlight_contribution = vec3(0, 0, 0);
	for (int i = 0; i < 4; i++) dirlight_contribution += DirLightContribution(i);
	vec3 pointlight_contribution = vec3(0, 0, 0);
	for (int i = 0; i < 4; i++) pointlight_contribution += PointLightContribution(i);

	frag_color = vec4(dirlight_contribution + pointlight_contribution, 1.0);
}
